from .base_test import BaseTest
import os

SAMPLE_DATASOURCE_PARAMS = {
    "description": "Python Integration Test Data Source. Delete if seen.",
    "title": "Python Integration Test Data Source",
    "source": "api",
    "config": '{"type":"json","method":"GET","transform":"products","url":"https://dummyjson.com/products","headers":"{}"}',
}


class TestDatasources(BaseTest):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.sampleDatasourceId = cls.onvoSDK.datasources.create(
            SAMPLE_DATASOURCE_PARAMS
        )[
            "id"
        ]  # Skipping test create

    def test_list(self):
        self.assertShouldRaise(None, self.onvoSDK.datasources.list)

    def test_get(self):
        self.assertShouldRaise(
            None, lambda: self.onvoSDK.datasources.get(self.sampleDatasourceId)
        )

    def test_initialize(self):
        self.assertShouldRaise(
            None, lambda: self.onvoSDK.datasources.initialize(self.sampleDatasourceId)
        )

    def test_update(self):
        self.assertShouldRaise(
            None,
            lambda: self.onvoSDK.datasources.update(
                self.sampleDatasourceId, {"title": "Renaming Test"}
            ),
        )

    # TODO: diagnose and fix 500: Internal server error
    # def test_upload_file(self):
    #     test_file_path = os.path.join(os.path.dirname(__file__), "python_onvo_test.csv")
    #     with open(test_file_path, "ab") as f:
    #         f.write(b"a,b,c\n1,2,3\n4,5,6\n")
    #     self.assertShouldRaise(
    #         None,
    #         lambda: self.onvoSDK.datasources.upload_file(
    #             self.sampleDatasourceId, test_file_path
    #         ),
    #     )
    #     os.remove(test_file_path)

    @classmethod
    def tearDownClass(cls) -> None:
        cls.onvoSDK.datasources.delete(cls.sampleDatasourceId)  # Skipping test delete
        super().tearDownClass()



from onvo.base_resource import BaseResource


class Accounts(BaseResource):
    """Endpoints to get Onvo accounts"""

    def list(self):
        """Get a list of all existing accounts in an organization

        Returns:
            list: A list of all existing accounts in an organization
        """
        return self.base_get("/accounts")

    def get(self, id):
        """Get details on a specific account

        Args:
            id (str): The account id to get details on

        Returns:
            dict: A dictionary containing details on the account
        """
        return self.base_get(f"/accounts/{id}")



from onvo.base_resource import BaseResource


class Dashboards(BaseResource):
    """Endpoints to manage Onvo dashboards"""

    def list(self):
        """Get a list of all dashboards under an organization

        Returns:
            list: A list of all existing dashboards
        """
        return self.base_get("/dashboards")

    def get(self, id):
        """Get details on a specific dashboard

        Args:
            id (str): The dashboard id to get details on

        Returns:
            dict: A dictionary containing details on the dashboard
        """
        return self.base_get(f"/dashboards/{id}")

    def create(self, data):
        """Create a new dashboard

        Args:
            data (dict[str, Any]): The data within the dashboard to be created

        Returns:
            dict: A dictionary containing the details of the newly created dashboard
        """
        return self.base_put(f"/dashboards", data=data)

    def update(self, id, data):
        """Update an existing dashboard

        Args:
            id (str): The dashboard id to update
            data (dict[str, Any]): The data within the dashboard to be updated

        Returns:
            dict: A dictionary containing the details of the updated dashboard
        """
        return self.base_post(f"/dashboards/{id}", data=data)

    def delete(self, id):
        """Delete an existing dashboard

        Args:
            id (str): The dashboard id to delete

        Returns:
            dict: A dictionary that shows status of the delete. Returns {"success": true} if successful.
        """
        return self.base_delete(f"/dashboards/{id}")

    def update_cache(self, id):
        """Update the cache of an existing dashboard

        Args:
            id (str): The dashboard id to update the cache of

        Returns:
            dict: A dictionary containing the details of the updated dashboard
        """
        return self.base_post(f"/dashboards/{id}/update-cache")



from onvo.base_resource import BaseResource


class Automations(BaseResource):
    """Endpoints to manage Onvo automations"""

    def list(self):
        """List all automations linked to an organization

        Returns:
            list: A list of all the automations
        """
        return self.base_get("/automations")

    def get(self, id):
        """Get details on a specific automation by id

        Args:
            id (str): The automation id to get details on

        Returns:
            dict: A dictionary containing details on the automation
        """
        return self.base_get(f"/automations/{id}")

    def get_runs(self, id):
        """Get all runs for an automation

        Args:
            id (str): The automation id to get runs for

        Returns:
            dict: A dictionary containing details on the automation
        """
        return self.base_get(f"/automations/{id}/runs")

    def create(self, data):
        """Create a new automation

        Args:
            data (dict[str, Any]): The data within the automation to be created

        Returns:
            dict: A dictionary containing the details of the newly created automation
        """
        return self.base_put(f"/automations", data=data)

    def update(self, id, data):
        """Update an existing automation

        Args:
            id (str): The automation id to update
            data (dict[str, Any]): The data within the automation to be updated

        Returns:
            dict: A dictionary containing the details of the updated automation
        """
        return self.base_post(f"/automations/{id}", data=data)

    def delete(self, id):
        """Delete an existing automation

        Args:
            id (str): The automation id to delete

        Returns:
            dict: A dictionary that shows status of the delete. Returns {"success": true} if successful.
        """
        return self.base_delete(f"/automations/{id}")



from onvo.base_resource import BaseResource
import re
import json


class Widgets(BaseResource):
    """Endpoints to manage Onvo widgets"""

    def __init__(self, endpoint, api_key, questions):
        super().__init__(endpoint, api_key)
        self.questions = questions

    def list(self, dashboard_id):
        """List out the widgets linked to a dashboard

        Args:
            dashboard_id (str): The dashboard id to list widgets for

        Returns:
            list: A list of all the widgets
        """
        return self.base_get("/widgets", params={"dashboard": dashboard_id})

    def get(self, id):
        """Get details on a specific widget

        Args:
            id (str): The widget id to get details on

        Returns:
            dict: A dictionary containing details on the widget
        """
        return self.base_get(f"/widgets/{id}")

    def export(self, id, format):
        """Export a widget

        Args:
            id (str): The widget id to export
            format (str): The export format to export the widget in. Must be one of: png, jpg, svg

        Returns:
            Any: The file in the requested format
        """
        return self.base_get(f"/widgets/{id}/export?format={format}")

    def create(self, dashboard_id: str, query: str):
        """Create a new widget. The endpoint first asks a question via the questions endpoint and uses the response to create a new widget

        Args:
            dashboard_id (str): The dashboard id to create the widget for
            query (str): The query string to ask a question

        Returns:
            dict: A dictionary containing the details of the newly created widget
        """
        conversation = self.questions.create(dashboard_id, query)
        question_text = conversation["messages"][-1]["content"]

        def extract_pattern(pattern):
            return re.findall(pattern, question_text)[0].strip()

        code = extract_pattern(r"```python([\S\n\t\v ]*)```\n```json")
        cache = extract_pattern(r"```json([\S\n\t\v ]*)```")

        return self.base_put(
            "/widgets",
            data={
                "dashboard": dashboard_id,
                "code": code,
                "cache": json.loads(cache),
                "messages": [{"role": "user", "content": query}],
            },
        )

    def update(self, id, data):
        """Update an existing widget. Note: the code is not re-run when using this endpoint.

        Args:
            id (str): The widget id to update
            data (dict[str, Any]): The data within the widget to be updated

        Returns:
            dict: A dictionary containing the details of the updated widget
        """
        return self.base_post(f"/widgets/{id}", data=data)

    def request_edit(self, id, data):
        """Request an edit to an existing widget. Note: the code will be rebuilt based on changes to the messages

        Args:
            id (str): The widget id to request an edit
            data (dict[str, Any]): The data within the widget to be updated

        Returns:
            dict: A dictionary containing the details of the updated widget
        """
        return self.base_patch(f"/widgets/{id}", data=data)

    def execute_code(self, id, code):
        """Execute code. The code given to this endpoint will be run in the context of the widget. The code in the widget will not be replaced. Use this endpoint as a test bed to verify that your edits to the widget work.

        Args:
            id (str): The widget id to execute the code in
            code (str): The code to execute in the widget

        Returns:
            dict: A dictionary containing the details of the updated widget
        """
        return self.base_post(f"/widgets/{id}/execute-code", data={"code": code})

    def delete(self, id):
        """Delete an existing widget

        Args:
            id (str): The widget id to delete

        Returns:
            dict: A dictionary that shows status of the delete. Returns {"success": true} if successful.
        """
        return self.base_delete(f"/widgets/{id}")



from .base_test import BaseTest


class TestTeams(BaseTest):

    def test_list(self):
        self.assertShouldRaise(None, self.onvoSDK.teams.list)

    def test_get(self):
        sample_id = self.onvoSDK.teams.list()[0]["id"]
        self.assertShouldRaise(None, lambda: self.onvoSDK.teams.get(sample_id))

    def test_update(self):
        sample_id = self.onvoSDK.teams.list()[0]["id"]
        self.assertShouldRaise(
            None,
            lambda: self.onvoSDK.teams.update(
                sample_id,
                {"name": "Onvo AI", "phone_number": "+91 807 506 5108"},
            ),
        )



from onvo.base_resource import BaseResource


class Questions(BaseResource):
    """Endpoints to manage Onvo questions"""

    def list(self, dashboard_id: str):
        """List all questions linked to a dashboard

        Args:
            dashboard_id (str): The dashboard id to list questions for

        Returns:
            list: A list of all the questions
        """
        return self.base_get("/questions", params={"dashboard": dashboard_id})

    def delete(self, id: str):
        """Delete an existing question

        Args:
            id (str): The question id to delete

        Returns:
            dict: A dictionary that shows status of the delete. Returns {"success": true} if successful.
        """
        return self.base_delete(f"/questions/{id}")

    def create(self, dashboard_id: str, query: str):
        """Create a new question

        Args:
            dashboard_id (str): The dashboard id to create the question for
            query (str): The query string to ask a question

        Returns:
            dict: A dictionary containing the details of the newly created question
        """
        return self.base_post(
            "/questions",
            data={
                "dashboard": dashboard_id,
                "messages": [{"role": "user", "content": query}],
            },
        )

    # TODO: create the update method once the postman shows viable example



from .base_test import BaseTest

SAMPLE_DASHBOARD_PARAMS = {
    "description": "Sample dashboard for widget testing",
    "title": "Widget Testing Dashboard",
}

SAMPLE_AUTOMATION_PARAMS = {
    "description": "A sample description",
    "email_format": "This is an automation from Onvo",
    "email_subject": "This is an automation from Onvo",
    "enabled": "false",
    "output_format": "link",
    "recipient_type": "internal",
    "recipients": [],
    "schedule": "",
    "timezone": "Asia/Calcutta",
    "title": "API datasource test",
}


class TestAutomations(BaseTest):

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.sample_dashboard_id = cls.onvoSDK.dashboards.create(
            SAMPLE_DASHBOARD_PARAMS
        )["id"]
        # Skipping test create
        cls.sample_automation_id = cls.onvoSDK.automations.create(
            {
                **SAMPLE_AUTOMATION_PARAMS,
                "dashboard": cls.sample_dashboard_id,
                "created_by": cls.sample_user_id,
            }
        )["id"]

    def test_list(self):
        self.assertShouldRaise(None, self.onvoSDK.automations.list)

    def test_get(self):
        self.assertShouldRaise(
            None, lambda: self.onvoSDK.automations.get(self.sample_automation_id)
        )

    def test_get_runs(self):
        self.assertShouldRaise(
            None, lambda: self.onvoSDK.automations.get_runs(self.sample_automation_id)
        )

    def test_update(self):
        self.assertShouldRaise(
            None,
            lambda: self.onvoSDK.automations.update(
                self.sample_automation_id,
                {"description": "A New Test Description. Delete if seen."},
            ),
        )

    @classmethod
    def tearDownClass(cls) -> None:
        cls.onvoSDK.dashboards.delete(cls.sample_dashboard_id)
        cls.onvoSDK.automations.delete(cls.sample_automation_id)  # Skipping test delete
        super().tearDownClass()



from onvo.resources.dashboard_resources.datasources import DashboardDatasources


class Dashboard:
    """The Onvo Dashboard object."""

    def __init__(self, dashboard_id: str, endpoint: str, api_key: str):
        """Intialize the dashboard object. Can be initialized using the onvo.dashboard() method to autofill all fields except the dashboard_id

        Args:
            dashboard_id (str): Takes the id of the dashboard to manipulate
            endpoint (str): Takes an endpoint such as "https://dashboard.onvo.ai/api".
            api_key (str): Takes an API key.
        """
        params = [dashboard_id, endpoint, api_key]

        self.datasources = DashboardDatasources(*params)



from onvo.base_resource import BaseResource


class Datasources(BaseResource):
    """Endpoints to manage Onvo datasources"""

    def list(self):
        """List all datasources in an organization

        Returns:
            list: A list of all existing datasources along with necessary details
        """
        return self.base_get("/datasources")

    def get(self, id):
        """Get details on a specific datasource

        Args:
            id (str): The datasource id to get details on

        Returns:
            dict: A dictionary containing details on the datasource
        """
        return self.base_get(f"/datasources/{id}")

    def create(self, data):
        """Create a new datasource

        Args:
            data (dict[str, Any]): The data within the datasource to be created

        Returns:
            dict: A dictionary containing the details of the newly created datasource
        """
        return self.base_put(f"/datasources", data=data)

    def upload_file(self, id, file_path):
        """Upload a file to a datasource

        Args:
            id (str): The datasource id to upload the file to
            file_path (str): The path of the file to be uploaded

        Returns:
            dict: A dictionary containing the details of the newly created datasource
        """
        return self.base_post(
            f"/datasources/{id}/upload-file", files={"file": open(file_path, "rb")}
        )

    def initialize(self, id):
        """Initialize a datasource

        Args:
            id (str): The datasource id to initialize

        Returns:
            dict: A dictionary containing the details of the initialized datasource
        """
        return self.base_get(f"/datasources/{id}/initialize")

    def update(self, id, data):
        """Update an existing datasource

        Args:
            id (str): The datasource id to update
            data (dict[str, Any]): The data within the datasource to be updated

        Returns:
            dict: A dictionary containing the details of the updated datasource
        """
        return self.base_post(f"/datasources/{id}", data=data)

    def delete(self, id):
        """Delete an existing datasource

        Args:
            id (str): The datasource id to delete

        Returns:
            dict: A dictionary that shows status of the delete. Returns {"success": true} if successful.
        """
        return self.base_delete(f"/datasources/{id}")



from onvo.base_resource import BaseResource


class Sessions(BaseResource):
    """Endpoints to get and update sessions. A session creates a link between a user under your organisation and a dashboard."""

    def list(self, dashboard_id: str):
        """List all sessions for a dashboard

        Args:
            dashboard_id (str): ID of the dashboard to list sessions for

        Returns:
            list: A list of all sessions for a dashboard
        """
        return self.base_get("/sessions", params={"parent_dashboard": dashboard_id})

    def revoke(self, dashboard_id: str):
        """Delete all sessions for a dashboard

        Args:
            dashboard_id (str): ID of the dashboard to delete all sessions for

        Returns:
            dict: A dictionary that shows status of deleting all sessions. Returns {"success": true} if successful.
        """
        return self.base_delete("/sessions", params={"parent_dashboard": dashboard_id})

    def upsert(self, dashboard_id: str, user_id: str):
        """If a session already exists, it is retrieved to save the user's changes and edits. If a session does not exist, one is created and is retrieved.

        Args:
            dashboard_id (str): ID of the dashboard to create a session for
            user_id (str): ID of the user to create a session for

        Returns:
            dict: A dictionary containing the details of the created/retrieved session
        """
        return self.base_post(
            "/sessions", data={"dashboard": dashboard_id, "user": user_id}
        )

    def delete(self, id):
        """Delete an existing session

        Args:
            id (str): The session id to delete

        Returns:
            dict: A dictionary that shows status of the delete. Returns {"success": true} if successful.
        """
        return self.base_delete(f"/sessions/{id}")



from onvo.resources.dashboard_resources.base_resource import DashboardBaseResource


class DashboardDatasources(DashboardBaseResource):
    """Endpoints to manage Onvo datasources linked to a dashboard"""

    def list(self):
        """List all datasources linked to a dashboard

        Returns:
            list: A list of all the datasources
        """
        return self.base_get(f"/dashboards/{self.dashboard_id}/datasources")

    def link(self, datasource_id):
        """Link a datasource to a dashboard

        Args:
            datasource_id (str): The datasource id to link

        Returns:
            dict: A dictionary containing the details of the new link between the dashboard and datasource
        """
        return self.base_put(
            f"/dashboards/{self.dashboard_id}/datasources",
            data={"datasourceId": datasource_id},
        )

    def unlink(self, datasource_id):
        """Unlink a datasource from a dashboard

        Args:
            datasource_id (str): The datasource id to unlink

        Returns:
            dict: A dictionary that shows status of the unlink. Returns {"success": true} if successful.
        """
        return self.base_delete(
            f"/dashboards/{self.dashboard_id}/datasources/{datasource_id}"
        )



import requests
from requests.utils import quote
import json


class BaseResource:
    """Defines common functionality among all the resources within Onvo"""

    def __init__(self, endpoint, api_key):
        """Initalize endpoint and API key for rest of the functions

        Args:
            endpoint (str): Endpoint to which all requests go to
            api_key (str): API key to verify user
        """
        self.endpoint = endpoint
        self.headers = {"x-api-key": api_key, "Content-Type": "application/json"}

    def handle_response(self, response):
        """Raises an error if status does not indicate success, returns the processed response body.

        Args:
            response (Response): A standard Response object from the requests library

        Returns:
            list/dict/str: Processes the response body as json or raw text if the JSON processing fails.
        """
        if response.status_code >= 400:
            print(response.text)
        response.raise_for_status()
        try:
            return response.json()
        except ValueError:
            return response.text

    def process_options(self, options):
        """Take the additional parameters for a request and process them before passing them forward

        Args:
            options (dict[str, Any]): A dictionary containing the keys "data" and "params" which are added to the body and query of the request respectively
        """
        if "data" in options:
            options["data"] = json.dumps(options["data"])
        elif "files" in options:
            del options["headers"]["Content-Type"]
        # Standardizing params to use '%20' instead of '+' to replace spaces for all SDK requests
        if "params" in options:
            options["params"] = "&".join(
                [f"{key}={quote(value)}" for key, value in options["params"].items()]
            )

    def base_request(self, handler, subdirectory: str, additional_options):
        """The commmon flow for all requests made through Onvo

        Args:
            handler (function): The function that makes the call to the endpoint
            subdirectory (str): The path to the resource(s) to be accessed
            additional_options (dict[str, Any]): Additional options such as body/query

        Returns:
            list/dict/str: returns a list/dict of the resource(s) or a direct string input in a few cases such as Questions and Widgets
        """
        url = f"{self.endpoint}{subdirectory}"
        options = {"headers": self.headers, **additional_options}
        self.process_options(options)

        response = handler(url, **options)
        return self.handle_response(response)

    def base_get(self, subdirectory: str, **options) -> list | dict | str:
        """The base request for the RESTful index and show actions

        Args:
            subdirectory (str): The path to the resource(s)

        Returns:
            list | dict | str: The response object from all possible get requests
        """
        return self.base_request(requests.get, subdirectory, options)

    def base_put(self, subdirectory: str, **options) -> dict:
        """The base request for the RESTful create action

        Args:
            subdirectory (str): The path to the resource

        Returns:
            dict: The body of the newly created object or a success/failure message
        """
        return self.base_request(requests.put, subdirectory, options)

    def base_post(self, subdirectory: str, **options) -> dict:
        """The base request for the RESTful update action

        Args:
            subdirectory (str): The path to the resource

        Returns:
            dict: The body of the newly updated object or a success/failure message
        """
        return self.base_request(requests.post, subdirectory, options)

    def base_patch(self, subdirectory: str, **options) -> dict:
        """The base request for the RESTful update action through the patch channel

        Args:
            subdirectory (str): The path to the resource

        Returns:
            dict: The body of the newly updated object or a success/failure message
        """
        return self.base_request(requests.patch, subdirectory, options)

    def base_delete(self, subdirectory: str, **options) -> dict:
        """The base request for the RESTful delete action

        Args:
            subdirectory (str): The path to the resource

        Returns:
            dict: A success/failure message on deletion
        """
        return self.base_request(requests.delete, subdirectory, options)



from onvo.base_resource import BaseResource


class Teams(BaseResource):
    """Endpoints to get and update Onvo teams. Note: endpoints are scoped by API key to the organisation the user belongs to"""

    def list(self):
        """Retrieve all organisations a user has access to.

        Returns:
            list: A list of all the teams in an organization
        """
        return self.base_get("/teams")

    def get(self, id):
        """This endpoint returns the details of a given organisation.

        Args:
            id (str): The team id to get details on

        Returns:
            dict: A dictionary containing details on the team
        """
        return self.base_get(f"/teams/{id}")

    def update(self, id, data):
        """This endpoint allows you to edit an organisation.

        Args:
            id (str): The team id to update
            data (dict[str, Any]): A dictionary containing team data to be updated

        Returns:
            dict: A dictionary containing details on the team
        """
        return self.base_post(f"/teams/{id}", data=data)



from onvo.base_resource import BaseResource


class EmbedUsers(BaseResource):
    """Endpoints to manage Onvo embed users"""

    def list(self):
        """List all embed users

        Returns:
            list: A list of all embed users
        """
        return self.base_get("/embed-users")

    def get(self, id):
        """Get details on a specific embed user by id

        Args:
            id (str): The embed user id to get details on

        Returns:
            dict: A dictionary containing details on the embed user
        """
        return self.base_get(f"/embed-users/{id}")

    def delete(self, id):
        """Delete an embed user

        Args:
            id (str): The embed user id to delete

        Returns:
            dict: A dictionary that shows status of the delete. Returns {"success": true} if successful.
        """
        return self.base_delete(f"/embed-users/{id}")

    def upsert(self, data):
        """Created or update an embed user

        Args:
            data (dict[str, Any]): The data within the embed user to be created / updated

        Returns:
            dict: A dictionary containing the details of the newly created / updated embed user
        """
        return self.base_post("/embed-users", data=data)

    def get_access_token(self, id):
        """Get access token for an embed user

        Args:
            id (str): The embed user id to get access token for

        Returns:
            dict: A dictionary containing the details of the access token
        """
        return self.base_get(f"/embed-users/#{id}/token")
